@@ -48,7 +48,7 @@ class SseqChart:
         self._agent : Any = None
         self._batched_messages : List[Dict[str, Any]] = []
         # type: ignore
-        self._update_keys : Set[str] = set()
+        self._update_keys : Dict[str, int] = {}
         self._global_fields_to_update : Set[str] = set()
         self._batched_messages_lock = threading.Lock()
 
@@ -319,21 +319,24 @@ class SseqChart:
     def _add_edge_to_delete(self, e : ChartEdge):
         self._add_delete_message(e)
 
-    def _add_batched_message(self, key : str, kwargs : Dict[str, Any]):
+    def _add_batched_message(self, key : str, kwargs : Dict[str, Any], replace=False):
+        """ If replace is False, then if key """
         if not self._initialized:
             return        
-        if key in self._update_keys:
+        if key in self._update_keys and not replace:
             return
         with self._batched_messages_lock:
-            self._add_batched_message_raw(key, kwargs)
+            self._add_batched_message_raw(key, kwargs, replace)
 
-    def _add_batched_message_raw(self, key : str, kwargs : Dict[str, Any]):
+    def _add_batched_message_raw(self, key : str, kwargs : Dict[str, Any], replace):
         # If we're actually bothering with locking we need to check again to make sure
         # key is not in dict to make sure that it didn't get inserted before we got the lock.
         if key in self._update_keys:
+            if replace:
+                self._batched_messages[self._update_keys[key]] = kwargs
             return
         if key is not None:       
-            self._update_keys.add(key)
+            self._update_keys[key] = len(self._batched_messages)
         self._batched_messages.append(kwargs)
 
     def _add_create_message(self, target_object : Any):
@@ -370,6 +373,34 @@ class SseqChart:
             )
         )
 
+    def _add_setting_message(self):
+        self._add_batched_message("settings", 
+            dict(
+                chart_id=self.uuid,
+                target_type=type(self).__name__,
+                command="update",
+                target_fields=self.get_settings(),
+            ),
+            replace=True
+        )
+
+    def get_settings(self):
+        return dict(
+            page_list=self._page_list,
+            x_projection=self.x_projection,
+            y_projection=self.y_projection,
+            x_range=self._x_range,
+            y_range=self._y_range,
+            # default_class_style=self.default_class_style,
+            # default_structline_style=self.default_structline_style,
+            # default_differential_style=self.default_differential_style,
+            # default_extension_style=self.default_extension_style,
+        )
+
+    def _clear_batched_messages(self):
+            self._batched_messages = []
+            self._update_keys = {}
+            
     async def update_a(self):
         """ Update the display. This will send a message to the display instructing it about how to 
             "catch up with" the current state of the SseqChart in the Python runtime.
@@ -383,8 +414,7 @@ class SseqChart:
                 return
             if self._agent:
                 await self._agent.send_batched_messages_a(self._batched_messages)
-            self._batched_messages = []
-            self._update_keys = set()
+            self._clear_batched_messages()
     
     def _normalize_class_argument(self, class_arg : ChartClassArg) -> ChartClass:
         """ If the argument is of type ChartClass, return it unmodified.
@@ -470,11 +500,11 @@ class SseqChart:
         """ The minimum x view extent. This represents the minimum x value that is possible to look at with the display.
             The display will not zoom or scroll left of this value.
         """
-        return self.x_range[0]
+        return self._x_range[0]
 
     @x_min.setter
     def x_min(self, value : int):
-        x_range = list(self.x_range)
+        x_range = list(self._x_range)
         x_range[0] = value
         self._x_range = tuple(x_range)
         self._add_setting_message()
@@ -484,11 +514,11 @@ class SseqChart:
         """ The maximum x view extent. This represents the maximum x value that is possible to look at with the display.
             The display will not zoom or scroll right of this value.
         """
-        return self.x_range[1]
+        return self._x_range[1]
 
     @x_max.setter
     def x_max(self, value : int):
-        x_range = list(self.x_range)
+        x_range = list(self._x_range)
         x_range[1] = value
         self._x_range = tuple(x_range)
         self._add_setting_message()
@@ -498,11 +528,11 @@ class SseqChart:
         """ The minimum y view extent. This represents the minimum y value that is possible to look at with the display.
             The display will not zoom or scroll below this value.
         """
-        return self.y_range[0]
+        return self._y_range[0]
     
     @y_min.setter
     def y_min(self, value : int):
-        y_range = list(self.y_range)
+        y_range = list(self._y_range)
         y_range[0] = value
         self._y_range = tuple(y_range)
         self._add_setting_message()
@@ -512,11 +542,11 @@ class SseqChart:
         """ The maximum y view extent. This represents the maximum y value that is possible to look at with the display.
             The display will not zoom or scroll above this value.
         """
-        return self.y_range[1]
+        return self._y_range[1]
 
     @y_max.setter
     def y_max(self, value : int):
-        y_range = list(self.y_range)
+        y_range = list(self._y_range)
         y_range[1] = value
         self._y_range = tuple(y_range)
         self._add_setting_message()
@@ -525,12 +555,12 @@ class SseqChart:
     @property
     def initial_x_min(self):
         """ The initial x minimum. When the display is first loaded this will be the smallest, leftmost visible x value."""        
-        return self.initial_x_range[0]
+        return self._initial_x_range[0]
 
 
     @initial_x_min.setter
     def initial_x_min(self, value : int):
-        initial_x_range = list(self.initial_x_range)
+        initial_x_range = list(self._initial_x_range)
         initial_x_range[0] = value
         self._initial_x_range = tuple(initial_x_range)
         self._add_setting_message()
@@ -538,11 +568,11 @@ class SseqChart:
     @property
     def initial_x_max(self):
         """ The initial x maximum. When the display is first loaded this will be the largest, rightmost visible x value."""        
-        return self.initial_x_range[1]
+        return self._initial_x_range[1]
 
     @initial_x_max.setter
     def initial_x_max(self, value : int):
-        initial_x_range = list(self.initial_x_range)
+        initial_x_range = list(self._initial_x_range)
         initial_x_range[1] = value
         self._initial_x_range = tuple(initial_x_range)
         self._add_setting_message()
@@ -550,12 +580,12 @@ class SseqChart:
     @property
     def initial_y_min(self):
         """ The initial y minimum. When the display is first loaded this will be the smallest, bottommost visible y value."""        
-        return self.initial_y_range[0]
+        return self._initial_y_range[0]
 
 
     @initial_y_min.setter
     def initial_y_min(self, value : int):
-        initial_y_range = list(self.initial_y_range)
+        initial_y_range = list(self._initial_y_range)
         initial_y_range[0] = value
         self._initial_y_range = tuple(initial_y_range)
         self._add_setting_message()
@@ -563,11 +593,11 @@ class SseqChart:
     @property
     def initial_y_max(self):
         """ The initial y maximum. When the display is first loaded this will be the largest, topmost visible y value."""        
-        return self.initial_y_range[1]
+        return self._initial_y_range[1]
 
     @initial_y_max.setter
     def initial_y_max(self, value : int):
-        initial_y_range = list(self.initial_y_range)
+        initial_y_range = list(self._initial_y_range)
         initial_y_range[1] = value
         self._initial_y_range = tuple(initial_y_range)
         self._add_setting_message()
