@@ -5,16 +5,116 @@ from ..helper_classes import (
 
 from ..infinity import INFINITY
 from uuid import uuid4
-from .chart_types import UUID_str, Color
-from .chart_shape import Shape
+from .display_primitives import UUID_str, Color, Shape
 from .chart_edge import ChartEdge
 
-from typing import TYPE_CHECKING, List, Any, Tuple, cast, Dict, Union, Optional
+from typing import TYPE_CHECKING, List, Any, Tuple, cast, Dict
 if TYPE_CHECKING:
     from ..chart import SseqChart
     from .chart_edge import ChartEdge
 
 
+class ChartClassStyle:
+    """ The data that determine the visual style of a class. """
+    def __init__(self,
+        group_name : str = "",
+        shape : Shape = Shape().circled(5),
+        background_color : Color = (0, 0, 0, 1),
+        border_color : Color = (0, 0, 0, 1),
+        foreground_color : Color = (0, 0, 0, 1),
+        border_width : float = 2,
+    ):
+        self._group_name = group_name
+        self._shape = shape
+        self._background_color = background_color
+        self._border_color = border_color
+        self._foreground_color = foreground_color
+        self._border_width = border_width
+
+    def to_json(self) -> Dict[str, Any]:
+        return dict(
+            type=type(self).__name__,
+            group_name = self._group_name,
+            shape = self._shape,
+            background_color = self._background_color,
+            border_color = self._border_color,
+            border_width = self._border_width,
+            foreground_color = self._foreground_color,
+        )
+
+    @staticmethod
+    def from_json(json : Dict[str, Any]) -> "ChartClassStyle":
+        assert json.pop("type") == ChartClassStyle.__name__
+        return ChartClassStyle(**json)
+
+    @property
+    def group_name(self) -> str:
+        """ The name of the "group" represented by the current glyph. This does not directly affect the rendered chart, 
+            it is intended for clerical purposes. If group_name is present it is treated as if it were unique by the ClassStyle repr.
+        """        
+        return self._group_name
+    
+    @group_name.setter
+    def group_name(self, v : str):
+        self._group_name = v
+
+    @property
+    def shape(self) -> Shape:
+        """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
+        return self._shape
+    
+    @shape.setter
+    def shape(self, v : Shape): # type: ignore
+        self._shape = v
+
+    @property
+    def background_color(self) -> Color:
+        """ The color to use to draw the shape background. """
+        return self._background_color
+    
+    @background_color.setter
+    def background_color(self, v : Color): # type: ignore
+        """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
+        self._background_color = v
+
+    @property
+    def border_color(self) -> PageProperty[Color]:
+        """ The color to use to draw the shape border. """
+        return self._border_color
+    
+    @border_color.setter
+    def border_color(self, v : Color): # type: ignore
+        """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
+        self._border_color = v
+
+    @property
+    def foreground_color(self) -> Color:
+        """ The color to use to draw the shape foreground. """
+        return self._foreground_color
+    
+    @foreground_color.setter
+    def foreground_color(self, v : Color): # type: ignore
+        """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
+        self._foreground_color = v
+
+    @property
+    def border_width(self) -> float:
+        """ The thickness to draw the shape border. """
+        return self._border_width
+    
+    @border_width.setter
+    def border_width(self, v : float): # type: ignore
+        """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
+        self._border_width = v
+
+    def __repr__(self):
+        if self.group_name:
+            return f"ClassStyle({self.group_name})"
+        else:
+            d = self.to_json()
+            del d["type"]
+            return f"ClassStyle({d})"
+
 
 class ChartClass:
     """ """
@@ -27,20 +127,41 @@ class ChartClass:
         self._edges : List[ChartEdge] = []
         self._uuid = str(uuid4())
 
-        self._name = PageProperty("", parent=self)
-        self._shape = PageProperty(Shape().circled(5), parent=self)
-        self._background_color = PageProperty((0, 0, 0, 1), parent=self)
-        self._border_color = PageProperty((0, 0, 0, 1), parent=self)
-        self._foreground_color = PageProperty((0, 0, 0, 1), parent=self)
-        self._border_thickness = PageProperty(3, parent=self)
-        self._scale = PageProperty(1, parent=self)
-        self._visible = PageProperty(True, parent=self)
-        self._x_nudge = PageProperty(0, parent=self)
-        self._y_nudge = PageProperty(0, parent=self)
+        # These values don't really matter, just need to initialize the PageProperties or set_style will raise.
+        self.group_name = ""
+        self.shape = Shape().circled(5)
+        self.background_color = (0, 0, 0, 1)
+        self.border_color = (0, 0, 0, 1)
+        self.foreground_color = (0, 0, 0, 1)
+        self.border_width = 2
+
+        self.name = ""
+        self.scale = 1
+        self.visible = True
+        self.x_nudge = 0
+        self.y_nudge = 0 
         self._user_data = SignalDict({}, parent=self)
 
+    def get_style(self, page : int) -> ChartClassStyle:
+        return ChartClassStyle(
+            shape=self.shape[page],
+            background_color=self.background_color[page],
+            border_color=self.border_color[page],
+            foreground_color=self.foreground_color[page],
+            border_width=self.border_width[page]
+        )
+
+    def set_style(self, page : slice, style : ChartClassStyle) -> "ChartClass":
+        self.shape[page] = style.shape
+        self.background_color[page] = style.background_color
+        self.border_color[page] = style.border_color
+        self.foreground_color[page] = style.foreground_color
+        self.border_width[page] = style.border_width
+        return self
+
     def _needs_update(self):
-        self._sseq._add_class_to_update(self)
+        if hasattr(self, "_sseq"):
+            self._sseq._add_class_to_update(self)
 
     @staticmethod
     def from_json(json : Dict[str, Any]) -> "ChartClass":
@@ -62,9 +183,9 @@ class ChartClass:
         shape : PagePropertyOrValue[Shape],
         background_color : PagePropertyOrValue[Color],
         border_color : PagePropertyOrValue[Color],
-        border_thickness : PagePropertyOrValue[float],
+        border_width : PagePropertyOrValue[float],
         foreground_color : PagePropertyOrValue[Color],
-        user_data : Dict[str, Any] = None
+        user_data : SignalDict[Any]
     ):
         self._uuid = uuid
         self._max_page = max_page
@@ -73,14 +194,15 @@ class ChartClass:
         self._shape = ensure_page_property(shape, parent=self) 
         self._background_color = ensure_page_property(background_color, parent=self)
         self._border_color = ensure_page_property(border_color, parent=self)
-        self._border_thickness = ensure_page_property(border_thickness, parent=self)
+        self._border_width = ensure_page_property(border_width, parent=self)
         self._foreground_color = ensure_page_property(foreground_color, parent=self)
         self._scale = cast(PageProperty[float], ensure_page_property(scale, parent=self)) 
         self._visible = cast(PageProperty[bool], ensure_page_property(visible, parent=self))
         self._x_nudge = cast(PageProperty[float], ensure_page_property(x_nudge, parent=self))
         self._y_nudge = cast(PageProperty[float], ensure_page_property(y_nudge, parent=self))
 
-        self._user_data = SignalDict(user_data, parent=self) # type: ignore
+        self._user_data = user_data # type: ignore
+        user_data.set_parent(self)
 
     def to_json(self) -> Dict[str, Any]:
         return dict(
@@ -93,7 +215,7 @@ class ChartClass:
             shape = self._shape,
             background_color = self._background_color,
             border_color = self._border_color,
-            border_thickness = self._border_thickness,
+            border_width = self._border_width,
             foreground_color = self._foreground_color,
             scale=self.scale,
             visible=self.visible,
@@ -102,7 +224,14 @@ class ChartClass:
             user_data=self.user_data
         )
 
-    def replace(self, **kwargs : Any):
+    def replace(self, 
+        style : ChartClassStyle = None,
+        shape : Shape = None,
+        background_color : Color = None,
+        border_color : Color = None,
+        border_width : float = None,
+        foreground_color : Color = None,
+    ) -> "ChartClass":
         """ If a class currently not a "permanent cycle" then set it to be a permanent cycle. \
             Takes keyword arguments to set the properties of the "replaced" class.
             For instance:
@@ -118,8 +247,17 @@ class ChartClass:
             raise ValueError("Class is already alive")
         page = self.max_page + 1
         self._max_page = INFINITY
-        for [key, value] in kwargs.items():
-            getattr(self, key)[page:] = value
+        if style:
+            self.set_style(slice(page, None), style)
+        for [key, value] in [
+            ["shape", shape], 
+            ["background_color", background_color], 
+            ["border_color", border_color],
+            ["foreground_color", foreground_color],
+            ["border_width", border_width]
+        ]:
+            if value:
+                getattr(self, key)[page:] = value
         self._needs_update()
         return self
 
@@ -179,7 +317,7 @@ class ChartClass:
     @max_page.setter
     def max_page(self, v : int):
         self._max_page = v
-        self._sseq._add_class_to_update(self)
+        self._needs_update()
 
     @property
     def name(self) -> PageProperty[str]:
@@ -191,6 +329,18 @@ class ChartClass:
         self._name = ensure_page_property(v, parent=self)
         self._needs_update()
 
+    @property
+    def group_name(self) -> PageProperty[str]:
+        """ The name of the "group" represented by the current class style. This does not directly affect the rendered chart, 
+            it is intended for clerical purposes.
+        """
+        return self._group_name
+    
+    @group_name.setter
+    def group_name(self, v : PagePropertyOrValue[str]):
+        self._group_name = ensure_page_property(v, parent=self)
+        self._needs_update()
+
     @property
     def shape(self) -> PageProperty[Shape]:
         """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
@@ -217,7 +367,7 @@ class ChartClass:
         """ The color to use to draw the shape border. """
         return self._border_color
     
-    @shape.setter
+    @border_color.setter
     def border_color(self, v : PagePropertyOrValue[Color]): # type: ignore
         """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
         self._border_color = ensure_page_property(v, parent=self)
@@ -235,14 +385,14 @@ class ChartClass:
         self._needs_update()
 
     @property
-    def border_thickness(self) -> PageProperty[float]:
+    def border_width(self) -> PageProperty[float]:
         """ The thickness to draw the shape border. """
-        return self._border_thickness
+        return self._border_width
     
-    @border_thickness.setter
-    def border_thickness(self, v : PagePropertyOrValue[float]): # type: ignore
+    @border_width.setter
+    def border_width(self, v : PagePropertyOrValue[float]): # type: ignore
         """ Control how to draw the class. Note that it is the responsibility of display implementations to handle these shapes."""
-        self._border_thickness = ensure_page_property(v, parent=self)
+        self._border_width = ensure_page_property(v, parent=self)
         self._needs_update()
 
     @property
@@ -272,11 +422,22 @@ class ChartClass:
         """ The x displacement of the class from its default position. """
         return self._x_nudge
 
+    @x_nudge.setter
+    def x_nudge(self, v : PagePropertyOrValue[float]): # type: ignore
+        self._x_nudge = ensure_page_property(v, parent=self)
+        self._needs_update()
+
+
     @property
     def y_nudge(self) -> PageProperty[float]:
         """ The y displacement of the class from its default position. """
         return self._y_nudge
 
+    @y_nudge.setter
+    def y_nudge(self, v : PagePropertyOrValue[float]): # type: ignore
+        self._y_nudge = ensure_page_property(v, parent=self)
+        self._needs_update()
+
     @property
     def user_data(self) -> SignalDict[Any]:
         """ Miscellaneous extra data that the user wants to add. All data added must be serializable. """
